<?php

declare(strict_types=1);

namespace Dedi\SyliusImportExportPlugin\EventListener;

use Dedi\SyliusImportExportPlugin\Exporter\GridAwareExporterInterface;
use Dedi\SyliusImportExportPlugin\Registry\GridExporterRegistry;
use Sylius\Component\Grid\Definition\Action;
use Sylius\Component\Grid\Definition\ActionGroup;
use Sylius\Component\Grid\Definition\Grid;
use Sylius\Component\Grid\Event\GridDefinitionConverterEvent;
use Symfony\Component\HttpFoundation\RequestStack;

class ExportButtonGridListener
{
    private readonly RequestStack $requeststack;

    private readonly GridExporterRegistry $gridExporterRegistry;

    public function __construct(Requeststack $requeststack, GridExporterRegistry $gridExporterRegistry)
    {
        $this->requeststack = $requeststack;
        $this->gridExporterRegistry = $gridExporterRegistry;
    }

    public function onSyliusGridAdmin(GridDefinitionConverterEvent $event): void
    {
        $grid = $event->getGrid();
        $exporters = $this->gridExporterRegistry->get($this->getGridCode($grid));

        if (\count($exporters) === 0) {
            return;
        }

        if (!$grid->hasActionGroup('main')) {
            $grid->addActionGroup(ActionGroup::named('main'));
        }

        $actionGroup = $grid->getActionGroup('main');

        if (!$actionGroup->hasAction('export')) {
            $action = Action::fromNameAndType('export', 'links');
            $action->setLabel('fos.import_export.ui.export');
            $action->setOptions([
                'class' => '',
                'icon' => 'save',
                'header' => [
                    'icon' => 'file code outline',
                    'label' => 'sylius.ui.type',
                ],
                'links' => [],
            ]);

            $actionGroup->addAction($action);
        }

        $action = $actionGroup->getAction('export');
        $options = $action->getOptions();
        $options['links'] = array_merge($options['links'], $this->createLinks($exporters));
        $action->setOptions($options);
    }

    /**
     * @return array[]
     */
    private function createLinks(array $exporters): array
    {
        $links = [];
        foreach ($exporters as $exporter) {
            $links[] = $this->addLink($exporter);
        }

        return $links;
    }

    private function addLink(GridAwareExporterInterface $exporter): array
    {
        $parameters = [
            'identifier' => $exporter->getIdentifier(),
        ];

        $request = $this->requeststack->getCurrentRequest();

        if (null !== $request) {
            $parameters['criteria'] = explode(',', $request->query->get('criteria', ''));
            /** @var array $routeParams */
            $routeParams = $request->attributes->get('_route_params', []);
            $parameters['criteria'] = array_merge(
                $parameters['criteria'],
                array_filter($routeParams, static fn ($key): bool => $key !== '_sylius', \ARRAY_FILTER_USE_KEY),
            );
        }

        return [
            'label' => $exporter->getLabel(),
            'icon' => 'file excel',
            'route' => 'app_admin_custom_export',
            'parameters' => $parameters,
        ];
    }

    private function getGridCode(Grid $grid): string
    {
        $code = $grid->getCode();

        // override grid code if we are on product grid filtered by taxon to avoid having to declare multiple services
        if ($code === 'sylius_admin_product_from_taxon') {
            return 'sylius_admin_product';
        }

        return $code;
    }
}
